-module(express).
-export([express/1]).

%
% express.erl
%
% Written by Matthew Sottile (matt@cs.uoregon.edu)
%


% given a list of characters, return a list of
% arities.
aritize([]) -> [];
aritize([H|T]) -> [gagenome:arity(H)]++(aritize(T)).

% given a level of arity values, figure out how many
% characters must live in the next level.
level_needs([]) -> 0;
level_needs([H|T]) -> H + level_needs(T).

% extract N elements from a list, returning the first N
% followed by the remainder.
extract_n(_,[]) -> {[],[]};
extract_n(N,[H|T]) -> 
  case N of
    0 -> {[],[H|T]};
    N -> {F,B} = extract_n(N-1,T),
         {[H|F],B}
  end.

% levelize a list of characters into the number of elements
% on each
int_levelize([],_) -> [];
int_levelize([H|T],REM) ->
  NEEDS = level_needs(aritize([H|T])),
  {F,B} = extract_n(NEEDS,REM),
  [length([H|T])]++(int_levelize(F,B)).

% given the int-levelized version of a list, return the list
% broken into levels.
stringize([],[]) -> [];
stringize(_,[]) -> [];
stringize(LIST, [F|B]) ->
  {FRONT,BACK} = extract_n(F,LIST),
  [FRONT]++(stringize(BACK,B)).
  
% creates a set of nodes as tuples containing tuples
makenodes([], _) -> [];
makenodes([H|T], [] ) ->
  AR = gagenome:arity(H),
  case AR of
    0 -> [H]++(makenodes(T,[]));
    _ -> io:format("ERROR1 (makenodes)!~n")
  end;
makenodes([H|T], KIDS ) ->
  AR = gagenome:arity(H),
  case AR of
    0 -> [H]++(makenodes(T,KIDS));
    1 -> [{H, hd(KIDS)}]++(makenodes(T,tl(KIDS)));
    2 -> [{H, hd(KIDS), hd(tl(KIDS))}]++(makenodes(T,tl(tl(KIDS))));
    _ -> io:format("ERROR2 (makenodes)!~n")
  end.

% convert the node set produced by stringize into a binary tree
nodify(NS,[]) -> NS;
nodify([],[H|T]) -> nodify(makenodes(H,[]), T);
nodify(NS,[H|T]) -> nodify(makenodes(H,NS), T).

% express a list into a tree structure
express([H|T]) -> 
 SLISTS = stringize([H|T], int_levelize([H],T)),
 nodify([], lists:reverse(SLISTS)).
