-module(gaeval).
-export([eval/2]).

%
% gaeval.erl
%
% Written by Matthew Sottile (matt@cs.uoregon.edu)
%

% evaluate a simple binary expression tree supporting +,-,*,/,sqrt, and two
% variables called 'a' and 'b'.
eval({UNOP, X}, Dict) ->
  Xval = eval(X,Dict),
  case UNOP of
    q when Xval>0.0 -> math:sqrt(Xval);
    _ -> throw({evalError, "Bad unop"})
  end;
eval({BINOP, X, Y}, Dict) ->
  Xval = eval(X,Dict),
  Yval = eval(Y,Dict),
  case BINOP of
    plus -> Xval+Yval;
    minus -> Xval-Yval;
    times -> Xval*Yval;
    divide when Yval /= 0.0 -> Xval/Yval;
    _ -> throw({evalError, "Bad binop"})
  end;
eval(VALUE, Dict) ->
  case VALUE of
    a -> {ok, V} = dict:find(a,Dict), V;
    b -> {ok, V} = dict:find(b,Dict), V;
    _ -> throw({evalError, "Bad value"})
  end.
