-module(gaops).
-export([one_pt_recomb/2, two_pt_recomb/2, mutate/2, breed/5]).

%
% gaops.erl
%
% Written by Matthew Sottile (matt@cs.uoregon.edu)
%

% one point recombination
one_pt_recomb(I1,I2) ->
  Pt = random:uniform(length(I1)),
  {I1F,I1B} = lists:split(Pt,I1),
  {I2F,I2B} = lists:split(Pt,I2),
  {I1F++I2B, I2F++I1B}.

% utility function to generate a random pair of different, ordered
% integers.
ordered_rand_pair(N) ->
  Pt1 = random:uniform(N),
  Pt2 = random:uniform(N),
  if 
    (Pt1 == Pt2) -> ordered_rand_pair(N);
    (Pt1 > Pt2) -> {Pt2,Pt1};
    true -> {Pt1,Pt2}
  end.

% two point recombination
two_pt_recomb(I1,I2) ->
  {Pt1,Pt2} = ordered_rand_pair(length(I1)),
  {I1F,I1B} = lists:split(Pt1,I1),
  {I2F,I2B} = lists:split(Pt1,I2),
  {I1BF,I1BB} = lists:split(Pt2-Pt1,I1B),
  {I2BF,I2BB} = lists:split(Pt2-Pt1,I2B),
  {I1F++I2BF++I1BB, I2F++I1BF++I2BB}.

% mutation, with head length specified to restrict tail mutations to
% terminals only.
mutate(I,Hlen) ->
  Mpt = random:uniform(length(I)),
  {_,Back} = lists:split(Mpt,I),
  {Front,_} = lists:split(Mpt-1,I),
  if
    (Mpt =< Hlen) -> NewChar = gagenome:random_any(),
                     Front++[NewChar]++Back;
    true -> NewChar = gagenome:random_terminal(),
                      Front++[NewChar]++Back
  end.

breed(A,B,Hlen,Pmut,P2PR) ->
  Mr = random:uniform(),
  M2pr = random:uniform(),
  if 
    (M2pr > P2PR) -> {X,_} = two_pt_recomb(A,B);
    true -> {X,_} = one_pt_recomb(A,B)
  end,
  if
    (Mr < Pmut) -> mutate(X,Hlen);
    true -> X
  end.